@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim:et sts=4 sw=4
 #
 # ibus-typing-booster - A completion input method for IBus
@@ -69,6 +68,7 @@ class SoundObject:
         self._wav_file: Optional[wave.Wave_read] = None
         self._paudio: Optional[pyaudio.PyAudio] = None
         self._play_pyaudio_thread: Optional[threading.Thread] = None
+        self._stop_event_paudio: Optional[threading.Event] = None
         self._simpleaudio_wave_o: Optional[simpleaudio.WaveObject] = None
         self._simpleaudio_play_o: Optional[simpleaudio.shiny.PlayObject] = None
         self._aplay_binary: Optional[str] = None
@@ -84,7 +84,7 @@ class SoundObject:
         if not os.access(self._path_to_sound_file, os.R_OK):
             LOGGER.info('Sound file %s not readable.', path_to_sound_file)
             return
-        if not self._requested_audio_backend in self._supported_audio_backends:
+        if self._requested_audio_backend not in self._supported_audio_backends:
             LOGGER.error('Audio backend %s not supported, use one of %s',
                          audio_backend, self._supported_audio_backends)
             return
@@ -132,7 +132,12 @@ class SoundObject:
         if not IMPORT_PYGAME_MIXER_SUCCESSFUL:
             return ''
         try:
-            pygame.mixer.init()
+            # pygame defaults to a low latency (small buffer) that is more
+            # appropriate for foreground games than background processes. Use a
+            # larger than default buffer to prevent audio issues in other
+            # processes when the system is under load:
+            # https://github.com/mike-fabian/ibus-typing-booster/issues/681
+            pygame.mixer.init(buffer=4096)
             if pygame.mixer.get_init():
                 pygame.mixer.music.load(self._path_to_sound_file)
                 return 'pygame'
@@ -145,7 +150,7 @@ class SoundObject:
     def _init_pyaudio(self) -> str:
         if not IMPORT_PYAUDIO_SUCCESSFUL:
             return ''
-        (mime_type, encoding) = mimetypes.guess_type(self._path_to_sound_file)
+        (mime_type, _encoding) = mimetypes.guess_type(self._path_to_sound_file)
         if mime_type not in ('audio/x-wav',):
             LOGGER.error(
                 'File %s has mime type %s and is not supported by simpleaudio',
@@ -154,7 +159,7 @@ class SoundObject:
         try:
             self._wav_file = wave.open(self._path_to_sound_file, 'rb')
             self._paudio = pyaudio.PyAudio()
-            self._stop_event_paudio: threading.Event = threading.Event()
+            self._stop_event_paudio = threading.Event()
             LOGGER.info('portaudio version = %s',
                         pyaudio.get_portaudio_version_text())
             return 'pyaudio'
@@ -167,7 +172,7 @@ class SoundObject:
     def _init_simpleaudio(self) -> str:
         if not IMPORT_SIMPLEAUDIO_SUCCESSFUL:
             return ''
-        (mime_type, encoding) = mimetypes.guess_type(self._path_to_sound_file)
+        (mime_type, _encoding) = mimetypes.guess_type(self._path_to_sound_file)
         if mime_type not in ('audio/x-wav',):
             LOGGER.error(
                 'File %s has mime type %s and is not supported by simpleaudio',
@@ -184,7 +189,7 @@ class SoundObject:
         return ''
 
     def _init_aplay(self) -> str:
-        (mime_type, encoding) = mimetypes.guess_type(self._path_to_sound_file)
+        (mime_type, _encoding) = mimetypes.guess_type(self._path_to_sound_file)
         if mime_type not in ('audio/x-wav',):
             LOGGER.error(
                 'File %s has mime type %s and is not supported by aplay',
@@ -214,13 +219,14 @@ class SoundObject:
             LOGGER.error('pyaudio.PyAudio() did not work.')
             return
         LOGGER.info('Playing sound with pyaudio ...')
+        chunk_size = 1024
         stream = self._paudio.open(
             format=self._paudio.get_format_from_width(
                 self._wav_file.getsampwidth()),
             channels=self._wav_file.getnchannels(),
             rate=self._wav_file.getframerate(),
-            output=True)
-        chunk_size = 1024
+            output=True,
+            frames_per_buffer=chunk_size)
         self._wav_file.rewind()
         data = self._wav_file.readframes(chunk_size)
         while data and not stop_event.is_set():
@@ -246,7 +252,8 @@ class SoundObject:
         LOGGER.info('Done playing sound with pyaudio.')
 
     def _play_pyaudio(self) -> None:
-        self._stop_event_paudio.clear()
+        if self._stop_event_paudio:
+            self._stop_event_paudio.clear()
         self._play_pyaudio_thread = threading.Thread(
             daemon=True,
             target=self._play_pyaudio_thread_function,
@@ -261,6 +268,8 @@ class SoundObject:
     def _stop_pyaudio(self) -> None:
         if not self._play_pyaudio_thread:
             return
+        if not self._stop_event_paudio:
+            return
         if (self._play_pyaudio_thread.is_alive()
             and not self._stop_event_paudio.is_set()):
             self._stop_event_paudio.set()
@@ -323,13 +332,14 @@ class SoundObject:
         if not self._aplay_binary:
             return
         try:
-            self._aplay_process = subprocess.Popen('aplay', shell=False,
-                                                   stdin=subprocess.PIPE,
-                                                   stderr=subprocess.PIPE,
-                                                   stdout=subprocess.PIPE,
-                                                   encoding=None,
-                                                   errors=None,
-                                                   text=None)
+            self._aplay_process = subprocess.Popen( # pylint: disable=consider-using-with
+                'aplay', shell=False,
+                stdin=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                encoding=None,
+                errors=None,
+                text=None)
         except (OSError, ValueError) as error:
             LOGGER.exception(
                 'cannot start aplay process %s: %s',
@@ -346,7 +356,7 @@ class SoundObject:
             return
         try:
             self._aplay_process.terminate()
-        except Exception as error:
+        except Exception as error: # pylint: disable=broad-except
             LOGGER.exception(
                 'cannot terminate aplay process %s: %s',
                 error.__class__.__name__, error)
@@ -354,10 +364,10 @@ class SoundObject:
                 LOGGER.info('Trying to kill aplay process')
                 self._aplay_process.kill()
                 LOGGER.info('aplay process killed')
-            except Exception as error:
+            except Exception as error2: # pylint: disable=broad-except
                 LOGGER.exception(
                     'cannot kill aplay process%s: %s',
-                    error.__class__.__name__, error)
+                    error.__class__.__name__, error2)
 
     def _play_aplay(self) -> None:
         self._play_aplay_thread = threading.Thread(
@@ -378,17 +388,17 @@ class SoundObject:
             and self._aplay_process.poll() is None):
             try:
                 self._aplay_process.terminate()
-            except Exception as error:
+            except Exception as error: # pylint: disable=broad-except
                 LOGGER.exception(
                 'cannot terminate aplay process %s: %s',
                 error.__class__.__name__, error)
                 try:
                     LOGGER.info('Trying to kill aplay process')
                     self._aplay_process.kill()
-                except Exception as error:
+                except Exception as error2: # pylint: disable=broad-except
                     LOGGER.exception(
                         'cannot kill aplay process%s: %s',
-                    error.__class__.__name__, error)
+                        error.__class__.__name__, error2)
         if self._play_aplay_thread.is_alive():
             self._play_aplay_thread.join(timeout=0.1)
             if self._play_aplay_thread.is_alive():
