@@ -27,6 +27,7 @@ from typing import NamedTuple
 from typing import Iterable
 from typing import Any
 import sys
+import re
 import ctypes
 import logging
 from gi import require_version # type: ignore
@@ -929,11 +930,15 @@ class TransliterationParts(NamedTuple):
     preedit:              The transliteration of the remaining input,
                           may still change by adding more input.
     cursor_pos: int       The cursor position in the preedit.
-                          Counted in codepoints, not glyphs.
-                          Usually this is at the end of the preedit
-                          but an input method may move the cursor
+                          Counted in codepoints, not glyphs (For example
+                          '☺\uFE0F' usually renders as one glyph (☺️) but
+                          has two codepoints).
+                          Usually this is at the end of the preedit but
+                          not always! An input method may move the cursor
                           within the preedit!
-                          (I think only ja-anthy.mim actually uses this)
+                          (I think only ja-anthy.mim actually moves the
+                          cursor within the preedit sometimes when moving
+                          or resizing the henkan region).
     status: str           May change for some input methods to
                           indicate a state.
                           For example in case of ja-anthy.mim,
@@ -941,6 +946,13 @@ class TransliterationParts(NamedTuple):
                           to '漢' in Henkan mode.
     candidates: List[str] May contain a list of candidates if the
                           input method can produce multiple candidates.
+    candidate_index: int  Index of the selected candidate.
+                          0 if no candidate is selected.
+    candidate_from: int   Start of the current candidate in the preedit.
+    candidate_to: int     End of the  current candidate in the preedit.
+                          If candidate_index == 0, i.e.
+                          no candidate is selected, the current candidate
+                          in the preedit is the first candidate.
     candidate_show: int   0: candidates should be hidden
                           1: candidates should be shown
     '''
@@ -950,17 +962,18 @@ class TransliterationParts(NamedTuple):
     cursor_pos: int = 0
     status: str = ''
     candidates: List[str] = []
+    candidate_index: int = 0
+    candidate_from: int = 0
+    candidate_to: int = 0
     candidate_show: int = 0
 
 class Transliterator:
-    # pylint: disable=line-too-long
     '''A class for transliterators using libm17n
 
     If initializing the transliterator fails, for example because a
     non-existing input method was given as the argument, a ValueError
     is raised.
     '''
-    # pylint: enable=line-too-long
     def __init__(self, ime: str) -> None:
         '''Initialize the input method to use for the transliteration
 
@@ -979,6 +992,8 @@ class Transliterator:
             return
         self._language = ime.split('-')[0]
         self._name = '-'.join(ime.split('-')[1:])
+        self._msymbol_single_char_with_prefix_pattern = re.compile(
+            r'^(?P<prefix>([SCMAGsH]-)*)(?P<unicode>.)$')
         self._im = libm17n__minput_open_im( # type: ignore
             libm17n__msymbol(ctypes.c_char_p(self._language.encode('utf-8'))), # type: ignore
             libm17n__msymbol(ctypes.c_char_p(self._name.encode('utf-8'))), # type: ignore
@@ -993,10 +1008,55 @@ class Transliterator:
         except ValueError as error: # NULL pointer access
             raise ValueError('minput_create_ic() failed') from error
 
+    def _convert_non_ascii_msymbol(self, msymbol: str) -> str:
+        # Python >= 3.7 has a str.isascii(), I could use that instead
+        # of my own is_ascii(), but that does not work on older
+        # distributions like openSUSE 15.6.
+        if itb_util.is_ascii(msymbol):
+            return msymbol
+        match = re.search(
+            self._msymbol_single_char_with_prefix_pattern, msymbol)
+        if not match:
+            return msymbol
+        name = IBus.keyval_name(IBus.unicode_to_keyval(match.group('unicode')))
+        return f'{match.group("prefix")}{name}'
+
+    def reset_ic(self) -> None:
+        '''Resets the input context
+
+        Can be used to reset the internal state of the input method to
+        the default.  For example, if zh-py is in fullwidth-mode (to
+        input fullwidth Latin), calling reset_ic() switches to the
+        default mode to input Chinese characters.
+        '''
+        if self._dummy:
+            return
+        libm17n__minput_reset_ic(self._ic) # type: ignore
+        # From the m17n-lib documentation:
+        #
+        # The minput_reset_ic () function resets input context $IC by
+        # calling a callback function corresponding to @b
+        # Minput_reset.  It resets the status of $IC to its initial
+        # one.  As the current preedit text is deleted without
+        # commitment, if necessary, call minput_filter () with the arg
+        # @b key #Mnil to force the input method to commit the preedit
+        # in advance.
+        #
+        # Looks like we need to do this here, i.e. call minput_filter
+        # with a final 'nil' msymbol to commit the preedit in the
+        # input context to make the next call to minput_reset_ic()
+        # work reliably.  Without that minput_reset_ic() sometimes
+        # segfaults.
+        _symbol = libm17n__msymbol(b'nil') # type: ignore
+        _retval = libm17n__minput_filter( # type: ignore
+            self._ic, _symbol, ctypes.c_void_p(None))
+
     def transliterate_parts(
             self,
             msymbol_list: Iterable[str],
-            ascii_digits: bool = False) -> TransliterationParts:
+            ascii_digits: bool = False,
+            reset: bool = False) -> TransliterationParts:
+        # pylint: disable=line-too-long
         '''Transliterate a list of Msymbol names
 
         :param msymbol_list: A list of strings which are interpreted
@@ -1410,24 +1470,20 @@ class Transliterator:
 
         For a test transliterating parts using 'ja-anthy' see 'tests/test_m17n_translit.py'.
         '''
+        # pylint: enable=line-too-long
         if not isinstance(msymbol_list, list):
             raise ValueError('Argument of transliterate() must be a list.')
         if self._dummy:
             return TransliterationParts(committed=''.join(msymbol_list),
                                        committed_index=len(msymbol_list))
-        libm17n__minput_reset_ic(self._ic) # type: ignore
+        if reset:
+            libm17n__minput_reset_ic(self._ic) # type: ignore
         committed = ''
         committed_index = 0
         preedit = ''
         candidates: List[str] = []
         for index, symbol in enumerate(msymbol_list):
-            if len(symbol) == 1 and not itb_util.is_ascii(symbol):
-                symbol = IBus.keyval_name(IBus.unicode_to_keyval(symbol))
-            elif (len(symbol) == 3 and symbol[1] == '-'
-                and symbol[0] in ('G', 'C', 'A')
-                and not itb_util.is_ascii(symbol[2])):
-                symbol = symbol[:2] + IBus.keyval_name(
-                    IBus.unicode_to_keyval(symbol[2]))
+            symbol = self._convert_non_ascii_msymbol(symbol)
             _symbol = libm17n__msymbol(symbol.encode('utf-8')) # type: ignore
             retval = libm17n__minput_filter( # type: ignore
                 self._ic, _symbol, ctypes.c_void_p(None))
@@ -1486,6 +1542,9 @@ class Transliterator:
             plist = libm17n__mplist_next(plist) # type: ignore
         cursor_pos = self._ic.contents.cursor_pos
         status = mtext_to_string(self._ic.contents.status)
+        candidate_index = self._ic.contents.candidate_index
+        candidate_from = self._ic.contents.candidate_from
+        candidate_to = self._ic.contents.candidate_to
         candidate_show = self._ic.contents.candidate_show
         # From the m17n-lib documentation:
         #
@@ -1508,6 +1567,22 @@ class Transliterator:
         # contents.  But apparently that final 'nil' is necessary to
         # make it work reliably. We can do this here because above we
         # read the preedit already and don’t need it anymore.
+        #
+        # It is not only necessary to make the next call to
+        # minput_reset_ic() work reliably, it is also necessary to
+        # commit any remaining preedit to avoid that the next
+        # transliteration starts with a non-empty preedit remaining
+        # from the previous transliteration.
+        #
+        # Unfortunately that makes state changing switches which
+        # affect only the next character not survive until the next
+        # transliteration.  For example when switching to
+        # single-fullwidth-mode by typing `Z` (see cjk-util.mim) with
+        # early commits (i.e. with the `tb:zh:py`), `aZ` will commit
+        # `啊`. The `Z` causes the commit but the state change done by
+        # the `Z` does not survive, typing `aZaZ` commits `啊啊`.
+        # With empty input one can type `Za` to get a single `ａ`
+        # FULLWIDTH LATIN SMALL LETTER A though.
         _symbol = libm17n__msymbol(b'nil') # type: ignore
         _retval = libm17n__minput_filter( # type: ignore
             self._ic, _symbol, ctypes.c_void_p(None))
@@ -1546,10 +1621,46 @@ class Transliterator:
         if preedit and not candidates and candidate_show:
             candidates = [preedit]
         if self._language == 'ja' and self._name == 'anthy':
-            # ja-anthy seems to produce a lot of useless geta marks '
-            # 〓'.  It also produces sometimes useless empty
-            # candidates and sometimes (when transliterating `1 ` even
-            # preedit='', cursor_pos=1. Try to fix this here:
+            # ja-anthy produces a lot of useless geta marks '〓'.  It
+            # also produces sometimes useless empty candidates and
+            # sometimes (when transliterating `1 `) even preedit='',
+            # cursor_pos=1.
+            #
+            # All these problems seem to be encoding problems caused
+            # by mimx-anthy.c in m17n-lib using EUC-JP encoding
+            # instead of UTF-8.  They all go away go away after fixing
+            # mimx-anthy.c to use UTF-8 by patching it like this:
+            #
+            # --- a/example/mimx-anthy.c
+            # +++ b/example/mimx-anthy.c
+            # @@ -129,15 +129,16 @@ new_context (MInputContext *ic)
+            #  {
+            #    AnthyContext *context;
+            #    anthy_context_t ac;
+            # -  MSymbol euc_jp = msymbol ("euc-jp");
+            # +  MSymbol utf_8 = msymbol ("utf-8");
+            #    /* Rebound to an actual buffer just before being used.  */
+            # -  MConverter *converter = mconv_buffer_converter (euc_jp, NULL, 0);
+            # +  MConverter *converter = mconv_buffer_converter (utf_8, NULL, 0);
+            #
+            #    if (! converter)
+            #      return NULL;
+            #    ac = anthy_create_context ();
+            #    if (! ac)
+            #      return NULL;
+            # +  anthy_context_set_encoding(ac, ANTHY_UTF8_ENCODING);
+            #    context = calloc (1, sizeof (AnthyContext));
+            #    context->ic = ic;
+            #    context->ac = ac;
+            #
+            # It is not necessary to use anthy-unicode (although this is
+            # probably better), I tested that this encoding problem here
+            # can be fixed by just patching mimx-anthy.c. Tested with
+            # anthy-9100h-56.fc41.x86_64
+            # anthy-unicode-1.0.0.20240502-8.fc41.x86_64
+            #
+            # For systems where mimx-anthy.c has not been fixed yet
+            # in m17n-lib, apply a runtime fix here:
             if preedit != '下駄':
                 candidates = [
                     candidate
@@ -1565,6 +1676,9 @@ class Transliterator:
                                        cursor_pos=cursor_pos,
                                        status=status,
                                        candidates=candidates,
+                                       candidate_index=candidate_index,
+                                       candidate_from=candidate_from,
+                                       candidate_to=candidate_to,
                                        candidate_show=candidate_show)
         return TransliterationParts(
             committed=convert_digits_to_ascii(committed),
@@ -1573,9 +1687,16 @@ class Transliterator:
             cursor_pos=cursor_pos,
             status=status,
             candidates=candidates,
+            candidate_index=candidate_index,
+            candidate_from=candidate_from,
+            candidate_to=candidate_to,
             candidate_show=candidate_show)
 
-    def transliterate(self, msymbol_list: Iterable[str], ascii_digits: bool = False) -> str:
+    def transliterate(
+            self,
+            msymbol_list: Iterable[str],
+            ascii_digits: bool = False,
+            reset: bool = False) -> str:
         '''Transliterate a list of Msymbol names
 
         :param msymbol_list: A list of strings which are interpreted
@@ -1658,7 +1779,7 @@ class Transliterator:
         'ඩනිෂ්ක නවීන් '
         '''
         transliteration_parts = self.transliterate_parts(
-            msymbol_list, ascii_digits)
+            msymbol_list, ascii_digits, reset)
         return transliteration_parts.committed + transliteration_parts.preedit
 
     def get_variables(self) -> List[Tuple[str, str, str]]:
