@@ -22,6 +22,9 @@ Main program of ibus-typing-booster
 '''
 from typing import Any
 from typing import Union
+from typing import Type
+from typing import Optional
+import types
 import os
 import sys
 import argparse
@@ -33,6 +36,8 @@ from signal import signal, SIGTERM, SIGINT
 from gi import require_version # type: ignore
 require_version('IBus', '1.0')
 from gi.repository import IBus # type: ignore
+require_version('Gio', '2.0')
+from gi.repository import Gio
 require_version('GLib', '2.0')
 from gi.repository import GLib
 # pylint: enable=wrong-import-position
@@ -42,6 +47,22 @@ import itb_version
 
 LOGGER = logging.getLogger('ibus-typing-booster')
 
+def log_unhandled_exception(
+    exc_type: Type[BaseException],
+    exc_value: BaseException,
+    exc_traceback: Optional[types.TracebackType]
+) -> None:
+    '''Log any unhandled exception.'''
+    if issubclass(exc_type, KeyboardInterrupt):
+        sys.__excepthook__(exc_type, exc_value, exc_traceback)
+        return
+    LOGGER.error(
+        'Unhandled exception',
+        exc_info=(exc_type, exc_value, exc_traceback)
+    )
+
+sys.excepthook = log_unhandled_exception
+
 DEBUG_LEVEL = int(0)
 try:
     DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
@@ -256,34 +277,8 @@ def write_xml() -> None:
     m17n_input_methods_skip = set([
         'NoIME',
     ])
-    # Input methods offering multiple candidates are not yet supported
-    # by typing-booster:
-    m17n_input_methods_multiple_candidates = set([
-        'ja-anthy',
-        't-lsymbol',
-        'ml-swanalekha',
-        'vi-han',
-        'vi-nomtelex',
-        'vi-nomvni',
-        'vi-tcvn',
-        'vi-telex',
-        'vi-viqr',
-        'vi-vni',
-        'zh-cangjie',
-        'zh-pinyin-vi',
-        'zh-pinyin',
-        'zh-py',
-        'zh-py-b5',
-        'zh-py-gb',
-        'zh-quick',
-        'zh-tonepy',
-        'zh-tonepy-gb',
-        'zh-tonepy-b5',
-        'zh-zhuyin',
-    ])
     supported_input_methods = sorted(
-        (m17n_input_methods - m17n_input_methods_skip)
-        - m17n_input_methods_multiple_candidates
+        m17n_input_methods - m17n_input_methods_skip
     ) + ['typing-booster']
 
     indic_languages = (
@@ -352,6 +347,12 @@ def write_xml() -> None:
         # https://bugzilla.redhat.com/show_bug.cgi?id=1076945
         # ibus-m17n uses rank 1 for the ar-kbd engine.
         (r'tb:ar:kbd', 2),
+        (r'tb:ja:anthy', -1), # ja:anthy is superseded by ibus-anthy
+        (r'tb:zh:py', -1), # zh:py is superseded by ibus-pinyin etc
+        (r'tb:zh:py-gb', -1), # zh:py-gb is superseded by ibus-pinyin etc
+        (r'tb:zh:py-b5', -1), # zh:py-b5 is superseded by ibus-pinyin etc
+        (r'tb:ko:han2', -1), # ko:han2 is superseded by ibus-hangul
+        (r'tb:ko:romaja', -1), # ko:romaja is superseded by ibus-hangul
     ]
     # Some Indic engines expect AltGr is automatically mapped
     ralt_switch_patterns = [
@@ -379,12 +380,20 @@ def write_xml() -> None:
         icon = os.path.join(ICON_DIR, 'ibus-typing-booster.svg')
         description = 'A completion input method to speedup typing.'
         symbol = '🚀'
-        setup = SETUP_TOOL
+        if (itb_util.is_desktop('gnome')
+            and itb_util.get_gnome_shell_version() >= (48, 3)):
+            # If running on Gnome and gnome-shell is new enough to contain
+            # https://gitlab.gnome.org/GNOME/gnome-shell/-/merge_requests/3753
+            # make the symbol black and white:
+            symbol = '🚀\uFE0E'
         icon_prop_key = 'InputMode'
         rank = 0
+        schema_path = '/org/freedesktop/ibus/engine/typing-booster/'
         if ime != 'typing-booster':
             m17n_lang, m17n_name = ime.split('-', maxsplit=1)
             name = f'tb:{m17n_lang}:{m17n_name}'
+            schema_path = ('/org/freedesktop/ibus/engine/tb/'
+                           f'{m17n_lang}/{m17n_name}/')
             rank = 0
             for pattern, pattern_rank in rank_patterns:
                 if re.fullmatch(pattern, name):
@@ -410,7 +419,14 @@ def write_xml() -> None:
             language = m17n_lang
             icon =  m17n_db_info.get_icon(ime)
             description = m17n_db_info.get_description(ime)
-            setup = SETUP_TOOL + f' --engine-name {name}'
+        setup = SETUP_TOOL + f' --engine-name {name}'
+        gsettings = Gio.Settings(
+            schema='org.freedesktop.ibus.engine.typing-booster',
+            path=schema_path)
+        user_symbol = itb_util.variant_to_value(
+            gsettings.get_user_value('inputmodetruesymbol'))
+        if user_symbol is not None:
+            symbol = user_symbol
 
         _name = SubElement(_engine, 'name')
         _name.text = name
@@ -483,7 +499,6 @@ def main() -> None:
     LOGGER.addHandler(log_handler)
     LOGGER.info('********** STARTING **********')
 
-    IBus.init()
     if _ARGS.daemon:
         if os.fork():
             sys.exit()
